import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

class LoadingButton extends StatefulWidget {
  final Future Function()? onPressed;
  final String text;

  const LoadingButton({Key? key, required this.onPressed, required this.text})
      : super(key: key);

  @override
  _LoadingButtonState createState() => _LoadingButtonState();
}

class _LoadingButtonState extends State<LoadingButton> {
  bool _isLoading = false;

  @override
  Widget build(BuildContext context) {
    return ElevatedButton(
      onPressed: (_isLoading || widget.onPressed == null) ? null : _loadFuture,
      child: _isLoading
          ? SizedBox(height: 16, width: 16, child: CircularProgressIndicator())
          : Text(widget.text),
    );
  }

  Future<void> _loadFuture() async {
    setState(() {
      _isLoading = true;
    });
    if (kDebugMode) {
      await widget.onPressed!();
    } else {
      try {
        await widget.onPressed!();
      } catch (e) {
        ScaffoldMessenger.of(context)
            .showSnackBar(SnackBar(content: Text('Error $e')));
      }
    }
    setState(() {
      _isLoading = false;
    });
  }
}
